/******************************************************************************
*******************       Interrupt functions/macros        *******************
*******************************************************************************

Macros which simplify access to interrupt enables, interrupt flags and
interrupt priorities. Increases code legibility.

******************************************************************************/

#define INT_ON   1
#define INT_OFF  0
#define INT_SET  1
#define INT_CLR  0

// Global interrupt enables
#define INT_GLOBAL_ENABLE(on) EA = (!!on)

#define DISABLE_ALL_INTERRUPTS() (IEN0 = IEN1 = IEN2 = 0x00)

#if(chip == 2430 || chip == 2431 || chip==2531)
#define INUM_RFERR 0
#endif
#if(chip == 1110 || chip == 1111 || chip == 2510 || chip == 2511)
#define INUM_RFTXRX 0
#endif
#define INUM_ADC   1
#define INUM_URX0  2
#define INUM_URX1  3
#define INUM_ENC   4
#define INUM_ST    5
#define INUM_P2INT 6
#define INUM_UTX0  7
#define INUM_DMA   8
#define INUM_T1    9
#define INUM_T2    10
#define INUM_T3    11
#define INUM_T4    12
#define INUM_P0INT 13
#define INUM_UTX1  14
#define INUM_P1INT 15
#define INUM_RF    16
#define INUM_WDT   17

#define NBR_OF_INTERRUPTS 18

// Macro used together with the INUM_* constants
// to enable or disable certain interrupts.
// Example usage:
//   INT_ENABLE(INUM_RFERR, INT_ON);
//   INT_ENABLE(INUM_URX0, INT_OFF);
//   INT_ENABLE(INUM_T1, INT_ON);
//   INT_ENABLE(INUM_T2, INT_OFF);
#if(chip == 2430 || chip == 2431 || chip == 2531)
#define INT_ENABLE(inum, on)                                                    \
   do {                                                                         \
      if      (inum==INUM_RFERR) { RFERRIE = on; }                              \
      else if (inum==INUM_ADC)   { ADCIE   = on; }                              \
      else if (inum==INUM_URX0)  { URX0IE  = on; }                              \
      else if (inum==INUM_URX1)  { URX1IE  = on; }                              \
      else if (inum==INUM_ENC)   { ENCIE   = on; }                              \
      else if (inum==INUM_ST)    { STIE    = on; }                              \
      else if (inum==INUM_P2INT) { (on) ? (IEN2 |= 0x02) : (IEN2 &= ~0x02); }   \
      else if (inum==INUM_UTX0)  { (on) ? (IEN2 |= 0x04) : (IEN2 &= ~0x04); }   \
      else if (inum==INUM_DMA)   { DMAIE   = on; }                              \
      else if (inum==INUM_T1)    { T1IE    = on; }                              \
      else if (inum==INUM_T2)    { T2IE    = on; }                              \
      else if (inum==INUM_T3)    { T3IE    = on; }                              \
      else if (inum==INUM_T4)    { T4IE    = on; }                              \
      else if (inum==INUM_P0INT) { P0IE    = on; }                              \
      else if (inum==INUM_UTX1)  { (on) ? (IEN2 |= 0x08) : (IEN2 &= ~0x08); }   \
      else if (inum==INUM_P1INT) { (on) ? (IEN2 |= 0x10) : (IEN2 &= ~0x10); }   \
      else if (inum==INUM_RF)    { (on) ? (IEN2 |= 0x01) : (IEN2 &= ~0x01); }   \
      else if (inum==INUM_WDT)   { (on) ? (IEN2 |= 0x20) : (IEN2 &= ~0x20); }   \
   } while (0)
#endif

#if(chip == 1110 || chip == 1111 || chip == 2510 || chip == 2511)
#define INT_ENABLE(inum, on)                                                    \
   do {                                                                         \
      if      (inum==INUM_RFTXRX) { RFTXRXIE = on; }                            \
      else if (inum==INUM_ADC)    { ADCIE   = on;  }                            \
      else if (inum==INUM_URX0)   { URX0IE  = on;  }                            \
      else if (inum==INUM_URX1)   { URX1IE  = on;  }                            \
      else if (inum==INUM_ENC)    { ENCIE   = on;  }                            \
      else if (inum==INUM_ST)     { STIE    = on;  }                            \
      else if (inum==INUM_P2INT)  { (on) ? (IEN2 |= 0x02) : (IEN2 &= ~0x02); }  \
      else if (inum==INUM_UTX0)   { (on) ? (IEN2 |= 0x04) : (IEN2 &= ~0x04); }  \
      else if (inum==INUM_DMA)    { DMAIE   = on;  }                            \
      else if (inum==INUM_T1)     { T1IE    = on;  }                            \
      else if (inum==INUM_T2)     { T2IE    = on;  }                            \
      else if (inum==INUM_T3)     { T3IE    = on;  }                            \
      else if (inum==INUM_T4)     { T4IE    = on;  }                            \
      else if (inum==INUM_P0INT)  { P0IE    = on;  }                            \
      else if (inum==INUM_UTX1)   { (on) ? (IEN2 |= 0x08) : (IEN2 &= ~0x08); }  \
      else if (inum==INUM_P1INT)  { (on) ? (IEN2 |= 0x10) : (IEN2 &= ~0x10); }  \
      else if (inum==INUM_RF)     { (on) ? (IEN2 |= 0x01) : (IEN2 &= ~0x01); }  \
      else if (inum==INUM_WDT)    { (on) ? (IEN2 |= 0x20) : (IEN2 &= ~0x20); }  \
   } while (0)
#endif


// Macro for setting interrupt group priority
// Example usage:
//   INT_PRIORITY(RFERR_RF_DMA, 3);
#define INT_PRIORITY(group, pri)                      \
   do {                                               \
      if (pri == 0) { IP0 &= ~group; IP1 &= ~group; } \
      if (pri == 1) { IP0 |=  group; IP1 &= ~group; } \
      if (pri == 2) { IP0 &= ~group; IP1 |=  group; } \
      if (pri == 3) { IP0 |=  group; IP1 |=  group; } \
   } while (0)
// Where pri is one of:
//   0 = Level 0 (lowest priority)
//   1 = Level 1
//   2 = Level 2
//   3 = Level 3 (highest priority)

// Where group is one of
#define RFERR_RF_DMA    0x01 // Group IP0
#define ADC_P2INT_T1    0x02 // Group IP1
#define URX0_UTX0_T2    0x04 // Group IP2
#define URX1_UTX1_T3    0x08 // Group IP3
#define ENC_P1INT_T4    0x10 // Group IP4
#define ST_WDT_P0INT    0x20 // Group IP5


// Macro used together with the INUM_* constants
// to read the interrupt flags.
// Example usage:
//   if (INT_GETFLAG(INUM_URX0))
//     ...
//   while (!INT_GETFLAG(INUM_URX0));

#if(chip == 2430 || chip == 2431 || chip == 2531)
#define INT_GETFLAG(inum) (                       \
   (inum==INUM_RFERR)       ? RFERRIF           : \
   (inum==INUM_ADC)         ? ADCIF             : \
   (inum==INUM_URX0)        ? URX0IF            : \
   (inum==INUM_URX1)        ? URX1IF            : \
   (inum==INUM_ENC)         ? ENCIF_0           : \
   (inum==INUM_ST)          ? STIF              : \
   (inum==INUM_P2INT)       ? P2IF              : \
   (inum==INUM_UTX0)        ? UTX0IF            : \
   (inum==INUM_DMA)         ? DMAIF             : \
   (inum==INUM_T1)          ? T1IF              : \
   (inum==INUM_T2)          ? T2IF              : \
   (inum==INUM_T3)          ? T3IF              : \
   (inum==INUM_T4)          ? T4IF              : \
   (inum==INUM_P0INT)       ? P0IF              : \
   (inum==INUM_UTX1)        ? UTX1IF            : \
   (inum==INUM_P1INT)       ? P1IF              : \
   (inum==INUM_RF)          ? S1CON &= ~0x03    : \
   (inum==INUM_WDT)         ? WDTIF             : \
   0                                              \
)
#endif
#if(chip == 1110 || chip == 1111 || chip == 2510 || chip == 2511)
#define INT_GETFLAG(inum) (                       \
   (inum==INUM_RFTXRX)      ? RFTXRXIF          : \
   (inum==INUM_ADC)         ? ADCIF             : \
   (inum==INUM_URX0)        ? URX0IF            : \
   (inum==INUM_URX1)        ? URX1IF            : \
   (inum==INUM_ENC)         ? ENCIF_0           : \
   (inum==INUM_ST)          ? STIF              : \
   (inum==INUM_P2INT)       ? P2IF              : \
   (inum==INUM_UTX0)        ? UTX0IF            : \
   (inum==INUM_DMA)         ? DMAIF             : \
   (inum==INUM_T1)          ? T1IF              : \
   (inum==INUM_T2)          ? T2IF              : \
   (inum==INUM_T3)          ? T3IF              : \
   (inum==INUM_T4)          ? T4IF              : \
   (inum==INUM_P0INT)       ? P0IF              : \
   (inum==INUM_UTX1)        ? UTX1IF            : \
   (inum==INUM_P1INT)       ? P1IF              : \
   (inum==INUM_RF)          ? S1CON &= ~0x03    : \
   (inum==INUM_WDT)         ? WDTIF             : \
   0                                              \
)

#endif


// Macro used to set or clear certain interrupt flags.
// Example usage:
//   INT_SETFLAG(INUM_URX0, INT_SET;
//   INT_SETFLAG(INUM_T3, INT_CLR);
#if(chip == 2430 || chip == 2431 || chip == 2531)
#define INT_SETFLAG(inum, f)                                                    \
   do {                                                                         \
      if      (inum==INUM_RFERR) { RFERRIF= f; }                                \
      else if (inum==INUM_ADC)   { ADCIF  = f; }                                \
      else if (inum==INUM_URX0)  { URX0IF = f; }                                \
      else if (inum==INUM_URX1)  { URX1IF = f; }                                \
      else if (inum==INUM_ENC)   { (f) ? (S0CON |= 0x03) : (S0CON &= ~0x03); }  \
      else if (inum==INUM_ST)    { STIF  = f;  }                                \
      else if (inum==INUM_P2INT) { P2IF  = f;  }                                \
      else if (inum==INUM_UTX0)  { UTX0IF= f;  }                                \
      else if (inum==INUM_DMA)   { DMAIF = f;  }                                \
      else if (inum==INUM_T1)    { T1IF  = f;  }                                \
      else if (inum==INUM_T2)    { T2IF  = f;  }                                \
      else if (inum==INUM_T3)    { T3IF  = f;  }                                \
      else if (inum==INUM_T4)    { T4IF  = f;  }                                \
      else if (inum==INUM_P0INT) { P0IF  = f;  }                                \
      else if (inum==INUM_UTX1)  { UTX1IF= f;  }                                \
      else if (inum==INUM_P1INT) { P1IF  = f;  }                                \
      else if (inum==INUM_RF)    { (f) ? (S1CON |= 0x03) : (S1CON &= ~0x03); }  \
      else if (inum==INUM_WDT)   { WDTIF = f;  }                                \
   } while (0)
#endif
#if(chip == 1110 || chip == 1111 || chip == 2510 || chip == 2511)
#define INT_SETFLAG(inum, f)                                                    \
   do {                                                                         \
      if      (inum==INUM_RFTXRX){ RFTXRXIF = f; }                              \
      else if (inum==INUM_ADC)   { ADCIF  = f; }                                \
      else if (inum==INUM_URX0)  { URX0IF = f; }                                \
      else if (inum==INUM_URX1)  { URX1IF = f; }                                \
      else if (inum==INUM_ENC)   { ENCIF_1 = ENCIF_0 = f; }                     \
      else if (inum==INUM_ST)    { STIF  = f;  }                                \
      else if (inum==INUM_P2INT) { P2IF  = f;  }                                \
      else if (inum==INUM_UTX0)  { UTX0IF= f;  }                                \
      else if (inum==INUM_DMA)   { DMAIF = f;  }                                \
      else if (inum==INUM_T1)    { T1IF  = f;  }                                \
      else if (inum==INUM_T2)    { T2IF  = f;  }                                \
      else if (inum==INUM_T3)    { T3IF  = f;  }                                \
      else if (inum==INUM_T4)    { T4IF  = f;  }                                \
      else if (inum==INUM_P0INT) { P0IF  = f;  }                                \
      else if (inum==INUM_UTX1)  { UTX1IF= f;  }                                \
      else if (inum==INUM_P1INT) { P1IF  = f;  }                                \
      else if (inum==INUM_RF)    { (f) ? (S1CON |= 0x03) : (S1CON &= ~0x03); }  \
      else if (inum==INUM_WDT)   { WDTIF = f;  }                                \
   } while (0)
#endif


